#include <stdio.h>
#include <strings.h>
#include "config.h"

char charset[1024];
char fontFilename[1024];
int textureSizeX=128, textureSizeY=128;
char modpath[1024];
bool DEBUG_MAPEDIT=false;

int nTranspLOD=0;
int transpLOD[50];

float transparencyLOD(int lvl){
  if (lvl<0) return 1.0;
  if (nTranspLOD==0) return 1.0;
  if (lvl>=nTranspLOD) return 1.0;
  return transpLOD[lvl]/100.0;
}

void skipLine(FILE *f){
  while (1) {
    char c=getc(f); 
    if (c=='\n') return;
    if (c==EOF) return;
  }
}

bool Separator(char c){
  switch (c) {
    case ' ': case '\t': case '\n': case 0: case EOF: case '"': return true;
    default: return false;
  }
}

bool setDefaults(){
  sprintf(charset,"abcdefghijklmnopqrstuvxyzABCDEFGHIJKLMNOPQRSTUVXYZ- ");
  sprintf(fontFilename,"usedFont.TTF");
  textureSizeX=1024;
  textureSizeY=512;
  sprintf(modpath,"C:\\Games\\Mount&Blade800\\Modules\\TLD\\");
  DEBUG_MAPEDIT=false;
}

char* readWord(FILE *f){
  static char res[1024];
  int i=0;
  while (1) {
    char c=getc(f);
    if (c==EOF && i==0) return NULL; 
    if (c=='#') { skipLine(f); continue; }
    if (c=='"') 
    { 
      if (i==0) while (1)
      { 
        c=getc(f); 
        if (c=='"') {res[i]=0; return res; }
        res[i++]=c; 
      }  
    }
    if (Separator(c)) {
      if (i>0 || c==EOF) {
        res[i]=0; break;
      }
    }
    else res[i++]=c;
  }
  return res;
}

void SaveConfig(char *filename){
  FILE *f=fopen(filename,"wt");
  fprintf(f,
"# MapScribbler config file!\n"
"\n"
"\n"
"# module path\n"
"module_path = \"%s\"\n"
"\n"
"# texture size to be used (default: 1024 x 512)\n"
"texture_size = %d x %d\n"
"\n"
"# used font (ant ttf file). Can include an absolute path\n"
"font = \"%s\"\n"
"\n"
"# charset - the set of chars you want to use in the labels. Must be ascii (sorry about that)\n"
"# naturally, the used font should include all used chars\n"
"charset = \"%s\"\n"
"\n"
"# test_mode - if set to 1, the exported stuff will be useable for debugging and testing\n"
"# purposes in MapEditor (but it will not be usable in M&&B!!)\n"
"test_mode = %d\n"  
  ,modpath
  ,textureSizeX, textureSizeY
  ,fontFilename
  ,charset
  ,(int)DEBUG_MAPEDIT
  );
}

bool ReadEqual(FILE *f){
  return (strcmp(readWord(f),"=")==0);
}

bool LoadConfig(char *filename){
  FILE *f=fopen(filename,"rt");
  
  int success=0;
  if (!f) {
    setDefaults();
    printf("Wrong or missing config file \"%s\"!\n",filename);
    //SaveConfig("tmp.txt");
    //printf("Saved defaults in %s... edit it and run me again...\n",filename);
    return false;
  }
  if (f) while (1)  {
   char *r=readWord(f);
   //printf("read '%s'",r);
   if (!r) break;
   if (strcmp(r,"module_path")==0) { 
      if (!ReadEqual(f)) break;
      sprintf(modpath,readWord(f));
      int s=strlen(modpath);
      if (s && (modpath[s-1]!='\\') && (modpath[s-1]!='/'))
      sprintf(modpath,"%s\\",modpath);
      success++;
   }
   if (strcmp(r,"texture_size")==0) { 
      if (!ReadEqual(f)) break;
      success+=fscanf(f," %d x %d",&textureSizeX, &textureSizeY);
   }
   
   if (strcmp(r,"font")==0) { 
      if (!ReadEqual(f)) break;
      sprintf(fontFilename,readWord(f));
      success++;
   }
   
   if (strcmp(r,"charset")==0) { 
      if (!ReadEqual(f)) break;
      sprintf(charset,readWord(f));
      success++;
   }
   
   if (strcmp(r,"test_mode")==0) {
      if (!ReadEqual(f)) break;
      int tmp;
      fscanf(f," %d",&tmp);
      DEBUG_MAPEDIT=(tmp==1);
   }
   
   if (strcmp(r,"mip_map")==0) {
      if (!ReadEqual(f)) break;
      int tmp; char c;
      if (!fscanf(f,"%c",&c)) break;
      while (1) {
        
        if (!fscanf(f," %d ",&tmp)) break;
        transpLOD[nTranspLOD++]=tmp;
        //printf("MIPmap Lvl %d: %d\n",nTranspLOD-1,transpLOD[nTranspLOD-1]);
        if (!fscanf(f,"%c",&c)) break;
        if (c==',') continue;
        if (c==']') break;
        return false;
      }
      success++;
   }
   
  }
  
  if (success==6) return true;
  printf("Error reading %s... try deleting it to reset the defaults...\n",filename);
  return false;
}
