
#include <vcg/space/point2.h>
#include <stdio.h>
#include <strings.h>

using namespace vcg;

#include "letter.h"
#include "texture.h"
#include "config.h"

bool Letter::IsSimilarTo(const Letter &l){
  if (l.c!=c) return false;
  if (l.italic!=italic) return false;
  if (fabs(size-l.size)>0.01) return false;
  //if (ori*l.ori<0.95) return false; ORIENTATION DOES NOT COUNT
  if (fabs(alpha-l.alpha)>4) return false;
  
  for (int i=0; i<4; i++) {
    if ((meshXY[i]-l.meshXY[i]).Norm()>0.01) return false;
    if (fabs(meshZ[i]-l.meshZ[i])>0.05) return false;
  }
  return true;
}

static float __max(float a, float b) { return (a<b)?b:a; }

void Letter::SetSimilarTo(Letter &l){
  original = &l;
  size = l.size = (size+l.size)/2;
  alpha = l.alpha = (alpha+l.alpha)/2;
  for (int i=0; i<4; i++) {
    meshZ[i] = l.meshZ[i] = __max(meshZ[i] , l.meshZ[i]);
  }
}

void SaveString(FILE *f, char *st){
  int x=strlen(st);
  fwrite(&x, 4, 1,  f);
  fwrite(st, 1, x, f);
}

void SaveInt(FILE *f, int x){
  fwrite(&x, 4, 1,  f);
}

void SaveUint(FILE *f, unsigned int x){
  fwrite(&x, 4, 1,  f);
}

void SaveFloat(FILE *f, float x){
  fwrite(&x, 4, 1,  f);
}


void SaveVert(FILE *f, float x,float y,float z){
  SaveFloat(f,x);
  SaveFloat(f,y);
  SaveFloat(f,z);
}

char* Letter::MeshIde(){
  static char res[255];
  sprintf(res,"map_str_%03d",index);
  return res;
}

void Letter::SaveIntoParties(FILE* f, int pos){
  unsigned int code;
  
#if MAB == 808
  code=0x01084400;
#else
  code=0x00084400;
#endif

  if (original==NULL)
    code |= mbindex;
  else
    code |= original->mbindex;
  
  
  float x=anchorPos.X(), y=anchorPos.Y();
  //float x=pos.X(), y=pos.Y();
#if (MAB==808)
  fprintf(f," 1 0 p_%s _ %u 0 0 3 0 0 0 0 0 %10.6f %10.6f %10.6f %10.6f %10.6f %10.6f 0.000000 0.000000 1.000000 1 96 500 0 0 \n"
              "0.5 0.0 0.0\n",MeshIde(),code, x,y,x,y,x,y);
#else
  fprintf(f," 1 %d %d p_%s _ %u 0 0 3 0 0 0 0 0 %10.6f %10.6f %10.6f %10.6f %10.6f %10.6f 0.0 0 \n"
              "%f\n",pos, pos, MeshIde(),code, x,y,x,y,x,y, 
              //float(3.14159265*3/2) // angle!
              float(angle) + float(3.14159265*3/2)
              );
#endif
            
}

void Letter::SaveIntoMapIcons(FILE* f, int pos){
  mbindex=pos;
#if MAB == 808
  fprintf(f,"%s 1 %s 1.000000 7\n",MeshIde(),MeshIde()); // 1 
#else
  fprintf(f,"%s %d %s %f 0 0 0 0 0\n\n\n",
    MeshIde(),
    1, // not animated
    MeshIde(),
    1.0f // scale
  );
#endif

}

vcg::Point2f Letter::DX() const{
  return ori*(size*Texture::multx(c)*width/100);
}

vcg::Point2f Letter::DY(int zeroone) const{
  vcg::Point2f res(ori.Y(),-ori.X());
  if (italic) res-=ori*0.29;
    
  return res*size * Texture::multy(c,zeroone) ;
}

// the following are defined in main (quick and dirty)
void GetMapClosestVert(vcg::Point2f p, vcg::Point2f& res, float& resz);
float GetMapZ(vcg::Point2f p); 
void GetMapN(float& nx,float& ny,float& nz);


void Letter::SetMeshPos(){


  meshXY[0]=-DX()+DY(0);
  meshXY[1]=-DX()-DY(1);
  meshXY[2]=DX()-DY(1);
  meshXY[3]=DX()+DY(0);

  float nx=0,ny=0,nz=1; 
    
  float basez=GetMapZ(pos);
  GetMapN(nx,ny,nz);
    
  bool safeAnchoring=false; 
  // if true, letter will be anchored in a safe position
  
  for (int i=0; i<4; i++) {
    // displacement to lay on ground
    float dzA=GetMapZ(pos+meshXY[i])-basez; 
    
    // displacement to follow normal
    float dzB=-(meshXY[i].X()*nx + meshXY[i].Y()*ny)/nz;
    
    // get the taller
    meshZ[i]=(dzA>dzB)?dzA:dzB;
    if ((meshZ[i]<=-0.005)||(meshZ[i]>=0.005)) safeAnchoring=true;
    
    meshZ[i]+=0.055; // to be on the safe side
    
    printf("Z=%f ",meshZ[i]);
  }
  
  angle = atan2(ori.Y() , ori.X() );

 // is this needed?
 /*
  if (safeAnchoring && 0) {
    float anchorZ; // z of the anchorpos
    // mode one: anchor on vertex
    GetMapClosestVert(pos, anchorPos, anchorZ);   
    // mode two: anchor on integer pos
    //vertPos[0]=(int)pos[0];vertPos[1]=(int)pos[1];anchorZ=GetMapZ(anchorPos);
  
    // correct pos for anchorpos
    for (int i=0; i<4; i++) {
      meshXY[i]+=pos-anchorPos;
      meshZ[i]+=basez-anchorZ;
    }
  } else anchorPos=pos;
  */
  anchorPos=pos;

#if (MAB == 808)
  // save letter as already 2D rotated
#else
  // save the letter without rotation (roation will be added in parties.txt
  vcg::Point2f tmp_ori = ori;
  ori = vcg::Point2f(1,0);
  meshXY[0]=-DX()+DY(0);
  meshXY[1]=-DX()-DY(1);
  meshXY[2]=DX()-DY(1);
  meshXY[3]=DX()+DY(0);
  ori= tmp_ori;   
#endif

}

void Letter::SaveIntoBRF(FILE *f){
  
  SaveString(f, MeshIde());
  SaveInt(f , 0);
  
  SaveString(f, "map_names"); // material used
    
  bool drawAnchor=false; //(safeAnchoring&&DEBUG_MAPEDIT); // also draw an visible anchor? (for debugging)
    
  SaveInt(f , 4+drawAnchor*5); // 4 vertices

  for (int i=0; i<4; i++) {
    SaveFloat(f, -meshXY[i].Y()); 
    SaveFloat(f,  meshXY[i].X()); 
    SaveFloat(f,  meshZ[i] );
  }
  if (drawAnchor) {
    SaveVert(f, 0,0,0);
    SaveVert(f, 0,-0.3,1);
    SaveVert(f, 0,+0.3,1);
    SaveVert(f, 0,-0.3,-1);
    SaveVert(f, 0,+0.3,-1);
  }
  
  SaveInt(f , 0); SaveInt(f , 0); SaveInt(f , 4+drawAnchor*5);
  float u[4]={Texture::u(c,1),Texture::u(c,1),Texture::u(c,0),Texture::u(c,0)}, 
        v[4]={Texture::v(c,0),Texture::v(c,1),Texture::v(c,1),Texture::v(c,0)};

  for (int k=0; k<4; k++) {
    SaveInt(f , k); 
    SaveUint(f , ((unsigned int)alpha)<<24); // color x vert! as 4 bytes AABBGGRR
    SaveFloat(f, 0);SaveFloat(f, 0);SaveFloat(f, 1); // normal x vert
    SaveFloat(f,u[k]);SaveFloat(f,1.0-v[k]); // tex coord x vert...
    SaveFloat(f,u[k]);SaveFloat(f,1.0-v[k]); // ... twice (2nd texture?)
  }
  
  if (drawAnchor) for (int k=0; k<5; k++) {
    SaveInt(f , k+4); 
    SaveUint(f , 0xFFFF0000); // color x vert! as 4 bytes AABBGGRR
    SaveFloat(f, 0);SaveFloat(f, 0);SaveFloat(f, 1); // normal x vert
    SaveFloat(f,0);SaveFloat(f,0); // tex coord x vert...
    SaveFloat(f,0);SaveFloat(f,0); // ... twice (2nd texture?)
  }
  
  SaveInt(f, 2+drawAnchor*4); // 2 tri-faces (1 quad)
  
  if (drawAnchor) {
    SaveInt(f, 4);SaveInt(f, 6);SaveInt(f, 5); // face (4,5,6)
    SaveInt(f, 4);SaveInt(f, 8);SaveInt(f, 7); // face (4,7,8)
    SaveInt(f, 4);SaveInt(f, 5);SaveInt(f, 6); // face (4,5,6)
    SaveInt(f, 4);SaveInt(f, 7);SaveInt(f, 8); // face (4,7,8)
  }
  
  SaveInt(f, 0);SaveInt(f, 2);SaveInt(f, 1); // face (0,2,1)
  SaveInt(f, 2);SaveInt(f, 0);SaveInt(f, 3); // face (2,0,3)

  
}

int globalIndex=0;
Letter::Letter(){index=globalIndex++;};
Letter::Letter(char _c){index=globalIndex++; c=_c;};

void Letter::ResetAll(){
  globalIndex=0;
}


#include <GL/gl.h>
#include <wrap/gl/space.h>

void Letter::Render(){
  glColor4ub(0,0,0,alpha);
  glBegin(GL_QUADS);

  vcg::Point2f orix=DX(), oriy0=DY(0), oriy1=DY(1);
  
  glTexCoord2f( Texture::u(c,0), Texture::v(c,0)); glVertex(pos+orix+oriy0);
  glTexCoord2f( Texture::u(c,1), Texture::v(c,0)); glVertex(pos-orix+oriy0);
  glTexCoord2f( Texture::u(c,1), Texture::v(c,1)); glVertex(pos-orix-oriy1);
  glTexCoord2f( Texture::u(c,0), Texture::v(c,1)); glVertex(pos+orix-oriy1);
  glEnd();
}
