#include <SDL/SDL.h>
#include <gl/gl.h>
#include <GL/glu.h>
#include <GL/openglut.h>
#include <SDL/SDL_image.h>

#include "map.h"

#include "letter.h"
#include "spline.h"

#include "texture.h"
#include "msgGUI.h"
#include "config.h"

std::vector<Spline> spline;
Spline newSpline;
bool editingNewSpline=false;
int curSpline=-1, curNode=-1;
bool wait4confirm=false, doneSomething=false;

/*
char* charset="abcdefghijklmnopqrstuvxyzABCDEFGHIJKLMNOPQRSTUVXYZ- ";
char* fontFilename="usedFont.TTF";
int textureSizeX=128, textureSizeY=128;
char* modpath="C:\\Games\\Mount&Blade800\\Modules\\TLD\\";
*/

vcg::Point2f clicked; // point clicked

typedef unsigned char byte;

typedef Uint32 Time;

int scrH=500, scrW=500; // altezza e larghezza viewport (in pixels)

float eyex=0.0, eyey=0.0,eyez=3.0;

double glmat0[16], glmat1[16]; int glviewp[4];

void GetGlStatus(){
  glGetDoublev(GL_PROJECTION_MATRIX, glmat0);
  glGetDoublev(GL_MODELVIEW_MATRIX, glmat1);
  glGetIntegerv(GL_VIEWPORT, glviewp);
}

vcg::Point2f GetPos(int x, int y){
  y=scrH-y;
  double rx,ry,rz ;
  vcg::Point2d res;
  gluUnProject(x,y,0,glmat1,glmat0,glviewp, &rx, &ry, &rz);
  return vcg::Point2f (rx,ry);
}

float GetMapZ(vcg::Point2f p){
  return map.GetZ(vcg::Point3f(p[0],p[1],0));
}

void GetMapClosestVert(vcg::Point2f p, vcg::Point2f& res, float& resz){
  vcg::Point3f pp=map.GetClosestVertP(vcg::Point3f(p[0],p[1],0) );
  res[0]=pp[0];
  res[1]=pp[1];
  resz=pp[2];
}


void GetMapN(float& nx,float& ny,float& nz){
  vcg::Point3f n= map.GetN();
  nx=n[0];
  ny=n[1];
  nz=n[2];
}


void redraw(){
  // ci automandiamo un messaggio che (s.o. permettendo)
  // ci fara' ridisegnare la finestra
  SDL_Event e;
  e.type=SDL_VIDEOEXPOSE;
  SDL_PushEvent(&e);
}

void initGl(){
  glDisable(GL_DEPTH_TEST);
  glFrontFace(GL_CW); // consideriamo Front FAcing le facce ClockWise
  glEnable(GL_POLYGON_OFFSET_FILL); // caro openGL sposta i 
                                    // frammenti generati dalla
                                    // rasterizzazione poligoni
  glPolygonOffset(1,1);             // indietro di 1
  
  //glDepthFunc(GL_LEQUAL);
  Texture::LoadGl();
}

void rendering(Time unused){
  
  // settiamo il viewport
  glViewport(0,0, scrW, scrH);
  
  glClearColor(0.5,0.5,0.5,1);
  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );


  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  float r=((float)scrW) / scrH;
  glOrtho( -r,+r,-1,+1,-100,+100); 
  
  if (wait4confirm) 
    Msg::ShowMiddle("Save Changes? ([Y]es,[N]o,[C]ancel)"); 
  else {
    glMatrixMode( GL_MODELVIEW ); 
    glLoadIdentity();
    glTranslatef(0,0,+.5);
    glScalef(-eyez,-eyez,1);
    glTranslatef(eyex,eyey,0);
    
    map.Render();
    
    GetGlStatus();
    glTranslatef(0,0,+10);
    if (editingNewSpline) newSpline.Render(true,3);
    for (int i=0; i<spline.size(); i++) spline[i].Render(curSpline==i, curNode);
    
    Msg::ShowHelp();
    Msg::Render();
    
    if (curSpline>=0) Msg::ShowStats(
      spline[curSpline].lettersize,
      spline[curSpline].alpha,
      spline[curSpline].width,
      spline[curSpline].italic
      );
  }

  glFinish();
  SDL_GL_SwapBuffers();
}

char* mypath(char* st){
  static char res[1024];
  sprintf(res,"%s%s",modpath,st);
  return res;
}

void SaveString(FILE *f, char *st);
void SaveInt(FILE *f, int x);
void SaveFloat(FILE *f, float x);


char* ReadLine(FILE *a){
  static char st[1024],c;
  int i=0;
  do {
    st[i++]=c=getc(a);
  }
  while (c!=EOF && c!='\n');
  if (c==EOF) return NULL;
  st[i-1]=0;
  return st;
}

bool CopyLine(FILE *a, FILE *b){
  char* st = ReadLine(a);
  if (!st) return false;
  fprintf(b,"%s\n",st);
  return true;
}

int NSplines(){
  return spline.size();
}

bool SaveInternal(){
  char *filename="savedData.txt";
  FILE *f=fopen(filename,"wt");
  if (!f) {
    Msg::Add("Cannot save internal file?!");
    return false;
  }
  fprintf(f,"v1\n");
  
  fprintf(f,"%d\n",spline.size());
  for (int i=0; i<spline.size(); i++) spline[i].Save(f);
  fclose(f);
      
  Msg::Add("All label saved. Will be loaded at the next run.");

  doneSomething=false;
  return true;
}

bool LoadInternal(){
  char *filename="savedData.txt";
  FILE *f=fopen(filename,"rt");
  Letter::ResetAll();
  if (!f) return false;
  int t;
  int ver;
  
  // detect version
  char c=getc(f); 
  if (c!='v') { ungetc(c,f); ver=0;}
  else fscanf(f,"%d\n",&ver);
  
  fscanf(f,"%d\n",&t);
  spline.resize(t);
  for (int i=0; i<t; i++) spline[i].Load(f,ver);
  fclose(f);
  doneSomething=false;
  return true;
}


void SaveBrfMaterials(FILE *f) {
#if (MAB == 808)
  const int N=16*9+1;
  byte materials[N]={
0x07,0x00,0x00,0x00,0x74,0x65,0x78,0x74,0x75,0x72,0x65,0x01,0x00,0x00,0x00,0x0D,
0x00,0x00,0x00,0x6D,0x61,0x70,0x5F,0x6E,0x61,0x6D,0x65,0x73,0x2E,0x64,0x64,0x73,
0x00,0xA0,0x0A,0x00,0x08,0x00,0x00,0x00,0x6D,0x61,0x74,0x65,0x72,0x69,0x61,0x6C,
0x01,0x00,0x00,0x00,0x09,0x00,0x00,0x00,0x6D,0x61,0x70,0x5F,0x6E,0x61,0x6D,0x65,
0x73,0x48,0x01,0x00,0x00,0x13,0x00,0x00,0x00,0x74,0x65,0x78,0x5F,0x6D,0x75,0x6C,
0x5F,0x63,0x6F,0x6C,0x6F,0x72,0x5F,0x61,0x6C,0x70,0x68,0x61,0x09,0x00,0x00,0x00,
0x6D,0x61,0x70,0x5F,0x6E,0x61,0x6D,0x65,0x73,0x04,0x00,0x00,0x00,0x6E,0x6F,0x6E,
0x65,0x04,0x00,0x00,0x00,0x6E,0x6F,0x6E,0x65,0x04,0x00,0x00,0x00,0x6E,0x6F,0x6E,
0x65,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,
  };
#else  
  const int N=16*9+9;
  byte materials[N]={
0x07,0x00,0x00,0x00,0x74,0x65,0x78,0x74,0x75,0x72,0x65,0x01,0x00,0x00,0x00,0x0D,
0x00,0x00,0x00,0x6D,0x61,0x70,0x5F,0x6E,0x61,0x6D,0x65,0x73,0x2E,0x64,0x64,0x73,
0x00,0xA0,0x0A,0x00,0x08,0x00,0x00,0x00,0x6D,0x61,0x74,0x65,0x72,0x69,0x61,0x6C,
0x01,0x00,0x00,0x00,0x09,0x00,0x00,0x00,0x6D,0x61,0x70,0x5F,0x6E,0x61,0x6D,0x65,
0x73,0x48,0x01,0x00,0x00,0x13,0x00,0x00,0x00,0x74,0x65,0x78,0x5F,0x6D,0x75,0x6C,
0x5F,0x63,0x6F,0x6C,0x6F,0x72,0x5F,0x61,0x6C,0x70,0x68,0x61,0x09,0x00,0x00,0x00,
0x6D,0x61,0x70,0x5F,0x6E,0x61,0x6D,0x65,0x73,0x04,0x00,0x00,0x00,0x6E,0x6F,0x6E,
0x65,0x04,0x00,0x00,0x00,0x6E,0x6F,0x6E,0x65,0x04,0x00,0x00,0x00,0x6E,0x6F,0x6E,
0x65,0x04,0x00,0x00,0x00,0x6E,0x6F,0x6E,0x65,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00 
  };

#endif
  fwrite(materials,N,1,f);
}

char* getAllLabelStr(){
  static char res[1024*64];
  res[0]=0;
  for (int i=0; i<spline.size(); i++) sprintf(res,"%s%s",res,spline[i].GetString());
  return res;
}

void SetMeshPos(){
  for (int i=0; i<spline.size(); i++) 
    for (int j=0; j<spline[i].letter.size(); j++) 
      if (spline[i].letter[j].c!=' ') 
        spline[i].letter[j].SetMeshPos();
}

int FindDuplicated(){
  int res=0;
  for (int ia=0; ia<spline.size(); ia++) 
    for (int ja=0; ja<spline[ia].letter.size(); ja++) 
      if (spline[ia].letter[ja].c!=' ')  {
        spline[ia].letter[ja].original = NULL;
        for (int ib=ia; ib<spline.size(); ib++) {
          for (int jb=((ia==ib)?ja+1:0); jb<spline[ib].letter.size(); jb++)  
            if (spline[ia].letter[ja].IsSimilarTo(spline[ib].letter[jb])) {
              spline[ia].letter[ja].SetSimilarTo(spline[ib].letter[jb]);
              res ++;
              break;
            }
          }
    }
  return res;
}

int TotalLetters(){
  int res=0;
  for (int i=0; i<spline.size(); i++) res+=spline[i].GetNIcons();
  return res;
}

bool SaveAll(){

  static char msg[512];  
  
  char* tmpst; int orig;
  FILE *fa,*fb, *f;

  SetMeshPos();
  
  // find and count duplicates
  int ndup = FindDuplicated();
  sprintf(msg,"Found %d duplicated",ndup);
  Msg::Add(msg); //return false;

  
  // count letters
  int nmesh=TotalLetters();
  
  nmesh-=ndup;

  Texture::Build(getAllLabelStr(),fontFilename,textureSizeX,textureSizeY);

  // brf file
  f=fopen(mypath("Resource\\map_names.brf"),"wb");  
  if (!f) Msg::Add("Problems exporting BRF file :( \n"); else {
    if (!DEBUG_MAPEDIT) SaveBrfMaterials(f); // comment for quick preview on mapEdit
    SaveString(f,"mesh");
    SaveInt(f,nmesh);  
    for (int i=0; i<spline.size(); i++) 
      for (int j=0; j<spline[i].letter.size(); j++) 
        if (spline[i].letter[j].c!=' ') if (spline[i].letter[j].original==NULL)
          spline[i].letter[j].SaveIntoBRF(f);  
    SaveString(f,"end");
    fclose(f);
    Msg::Add("(1/4) Exported resource file \"Resource\\map_names.brf\"! (add it to module.ini)\n");
  }

  // map_icons.txt
  fa=fopen(mypath("map_icons.orig.txt"),"rt");
  fb=fopen(mypath("map_icons.txt"),"wt");
  if (!fa || !fb) Msg::Add("Problems exporting map_icons.txt :( \n"); else {
  
    CopyLine(fa,fb);
    tmpst=ReadLine(fa); sscanf(tmpst,"%d", &orig);
    fprintf(fb,"%d\n",orig+nmesh);
    while (CopyLine(fa,fb));
    int pos=orig;
    for (int i=0; i<spline.size(); i++) 
      for (int j=0; j<spline[i].letter.size(); j++) 
        if (spline[i].letter[j].c!=' ') if (spline[i].letter[j].original==NULL) 
          spline[i].letter[j].SaveIntoMapIcons(fb,pos++);
    fclose(fa);
    fclose(fb);  
 
    sprintf(msg,"(2/4) Exported \"map_icons.txt\" file! (%d new icons, %d in total)",pos-orig, pos);
    if (pos>255) sprintf(msg,"%s WARNING: #icons = %d>255 - M&B will complain!!",msg,pos);
    Msg::Add(msg);
  }
  
  
  
  // parties.txt
  fa=fopen(mypath("parties.orig.txt"),"rt");
  fb=fopen(mypath("parties.txt"),"wt");
  if (!fa || !fb) Msg::Add("Problems exporting parties.txt :( \n"); else {
    
    CopyLine(fa,fb);
    tmpst=ReadLine(fa); sscanf(tmpst,"%d", &orig);
    fprintf(fb,"%d %d\n",orig+nmesh,orig+nmesh);
    for (int i=0; i<orig*2; i++) CopyLine(fa,fb);
    int pos=orig;
    for (int i=0; i<spline.size(); i++) 
      for (int j=0; j<spline[i].letter.size(); j++) 
        if (spline[i].letter[j].c!=' ') spline[i].letter[j].SaveIntoParties(fb,pos++);
    fclose(fa);
    fclose(fb);
    sprintf(msg,"(3/4) Exported \"parties.txt\" file! (%d new objects, %d in total)\n",pos-orig, pos);
    Msg::Add(msg);
  }

  // parties.txt
  //Texture::SaveAsPam( mypath("textures\\map_names_2.txt") );
  //if (Texture::SaveAsPng(mypath("textures\\map_names.png"))) {
   // Msg::Add("(4/4) Exported \"textures\\map_names.png\" texture! (convert it to .dds - use DXT3)\n");
  //}
  
  
  if (Texture::SaveAsDXT3(mypath("textures\\map_names.dds"))) {
    sprintf(msg,"(4/4) Exported \"textures\\map_names.dds\" DXT3 %dx%d texture! (%d letters)\n",
      textureSizeX, textureSizeY, Texture::nletters);
    Msg::Add(msg);
  } else {
    Msg::Add("(4/4) Problems exporting texture file?!? :( \n");
  }
  
  Texture::Build(charset,fontFilename,textureSizeX,textureSizeY);

  return true;
  

}

void SelectSpline(vcg::Point2f x){
  float bestDist=1.0;
  
  if (curSpline!=-1) {
    curNode=-1;
    for (int i=0; i<4; i++) {
      float dist=spline[curSpline].NodeDist(x,i);
      if (dist<bestDist)  {
        bestDist=dist;
        curNode=i;
      }
    }
  }

  int oldCurSpline=curSpline;
  if (curNode==-1) curSpline=-1;
  
  for (int i=0; i<spline.size(); i++) {
    float dist=spline[i].Dist(x);
    if (dist<bestDist) {
      bestDist=dist;
      curSpline=i;
      if (i!=oldCurSpline) curNode=-1;
    }
  }
}

bool ExitProgram(){
  if (!doneSomething) return true;
  wait4confirm=true;
  //Msg::Add("Save Changes? ([Y]es,[N]o,[C]ancel)");
  return false;
}

void CenterViewOn(vcg::Point2f p, float radius=0){
   eyex=-(p.X()-map.bbox.Center().X()) *(2.0/map.bbox.Diag());
   eyey=-(p.Y()-map.bbox.Center().Y()) *(2.0/map.bbox.Diag());
   if (radius) eyez=0.4*map.bbox.Diag()/radius;
}

void cleanRedrawEvent(){
  static SDL_Event buffer[20];
  int i=SDL_PeepEvents(buffer,20,SDL_GETEVENT,SDL_VIDEOEXPOSE);
}

int main(int argc, char *argv[]){
  if (!LoadConfig("MapScribbler.cfg")) return 0;
  

  LoadInternal();
  
  if (argc>1) if (strcmp(argv[1],"-b")==0) {
    SaveAll();
    Msg::DumpLog();
    return 0;
  }

  if (!Texture::Build(charset,fontFilename,textureSizeX,textureSizeY)) return 0;

  if (!map.ReloadMapColors()) map.SaveMapColors();
  
  if (!map.Load(mypath("map.txt"))) return 0;

  if (!map.LoadLocations(mypath("parties.orig.txt"))) {
    printf("Cannot find parties.orig.txt.\nRead the readme!\n");
    return 0;
  };

  SDL_Init( SDL_INIT_VIDEO ); 
  SDL_WM_SetCaption("MapScribbler v0.6","MapScribbler v0.6");
  SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );    
  SDL_SetVideoMode(scrW, scrH, 0, SDL_OPENGL|SDL_RESIZABLE);
  SDL_EnableUNICODE(1);
  SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);

  initGl();

  Msg::Add("MapScribbler v0.6, a Mound&Blade map labeller!");
  Msg::Add("by Marco Tarini 2007-2009");

  bool done=false; bool ctrl;
  while (!done) {
    
    SDL_Event e;
    
    if (SDL_WaitEvent(&e)) {
     switch (e.type) {

      case SDL_KEYDOWN:
        if ( (e.key.keysym.sym!=SDLK_LSHIFT)&&
             (e.key.keysym.sym!=SDLK_RSHIFT)&&
             (e.key.keysym.sym!=SDLK_LCTRL)&&
             (e.key.keysym.sym!=SDLK_RCTRL) ) redraw();
        if (wait4confirm) {
          if (e.key.keysym.unicode=='y' || e.key.keysym.unicode=='Y') {
            SaveInternal();
            done=true;
          }
          if (e.key.keysym.unicode=='n' || e.key.keysym.unicode=='N' 
             || e.key.keysym.sym==SDLK_ESCAPE) done=true;
          
          if (e.key.keysym.unicode=='c' || e.key.keysym.unicode=='C') {
            wait4confirm=false;
            Msg::Add("(quit canceled)");
          }
          break;
        } 
        //if (e.key.keysym.sym==SDLK_F11) {
        //  map.ShrinkRivers();
        //}
        if (e.key.keysym.sym==SDLK_F1) Msg::helpOn=!Msg::helpOn;
        if (e.key.keysym.sym==SDLK_F12) {
          //map.Save(mypath("map.txt"));
          SaveAll();
        }
        
        if (e.key.keysym.sym==SDLK_F2) SaveInternal();
        
        if ((e.key.keysym.sym==SDLK_TAB) && spline.size()) {
          if (SDL_GetModState()&KMOD_SHIFT) curSpline=(curSpline+spline.size()-1)%spline.size();
          else curSpline=(curSpline+1)%spline.size();
          CenterViewOn(spline[curSpline].Center(),spline[curSpline].GetRadius());
        }
        
        if (e.key.keysym.sym==SDLK_RETURN) { 
          curSpline=-1;
        }
        

        //if (e.key.keysym.sym==SDLK_s) eyey+=0.005;
        //if (e.key.keysym.sym==SDLK_w) eyey-=0.005;
        //if (e.key.keysym.sym==SDLK_a) eyex+=0.005;
        //if (e.key.keysym.sym==SDLK_d) eyex-=0.005;

        if (e.key.keysym.sym==SDLK_F8) map.ReloadMapColors();

        
        if (e.key.keysym.sym==SDLK_UP )   eyey+=2.0*40/(eyez*scrH);
        if (e.key.keysym.sym==SDLK_DOWN ) eyey-=2.0*40/(eyez*scrH);
        if (e.key.keysym.sym==SDLK_LEFT ) eyex-=2.0*40/(eyez*scrH);
        if (e.key.keysym.sym==SDLK_RIGHT) eyex+=2.0*40/(eyez*scrH);
        
        if (e.key.keysym.sym==SDLK_ESCAPE) {
          if (ExitProgram()) done=true;
        }
        
        ctrl=e.key.keysym.mod& (KMOD_LCTRL|KMOD_RCTRL);
      
        if (curSpline!=-1)  {
          if (e.key.keysym.sym==SDLK_PAGEDOWN ) {
            spline[curSpline].IncrAlpha( 5);
            doneSomething=true;
          }
          if (e.key.keysym.sym==SDLK_PAGEUP ) {
            spline[curSpline].IncrAlpha(-5);
            doneSomething=true;
          }

          if (e.key.keysym.sym==SDLK_BACKSPACE || e.key.keysym.sym==SDLK_DELETE) {
             if (!(spline[curSpline].RemoveLetter())) {
                std::vector<Spline>::iterator i=spline.begin();
                spline.erase(i+curSpline);
                curSpline=-1;
             }
             doneSomething=true;
          }
          short int key=e.key.keysym.unicode;
          if (ctrl) {
            if (e.key.keysym.sym==SDLK_i) {
              spline[curSpline].italic=!spline[curSpline].italic;
              spline[curSpline].UpdateLetterStats();
              doneSomething=true;
            }       
            if (e.key.keysym.sym==SDLK_f) {
              spline[curSpline].Flip();
              curNode=-1;
              doneSomething=true;
            }  
            
            if (e.key.keysym.sym==SDLK_p) {
              spline[curSpline].ChangeAllCaps();
              curNode=-1;
              doneSomething=true;
            }  
            // cut and paste of stats
            static Spline clipboard; 
            static bool clipboardPresent=false;
            if (e.key.keysym.sym==SDLK_c) {
              clipboardPresent=true;
              clipboard.CopyStats(spline[curSpline]);
            }
            if (e.key.keysym.sym==SDLK_v) {
              if (clipboardPresent)
              spline[curSpline].CopyStats(clipboard);
              spline[curSpline].UpdateLetterStats();
            }
            

          } 
          if (Texture::CanUseChar(key)) {
            spline[curSpline].AddLetter(key);
            doneSomething=true;
          }
        }

        if (ctrl && e.key.keysym.sym==SDLK_d) {
              SetMeshPos();
              int k=FindDuplicated();
              int h=TotalLetters();
              char msg[1024];
              sprintf(msg, "In current config, %d letters out of %d can use duplicates",k,h);
              Msg::Add(msg);
        }

        break;
      case SDL_KEYUP:
        break;
      case SDL_QUIT:  
        if (wait4confirm) done=true;  
        else if (ExitProgram()) done=true; else redraw();   
        break;
      case SDL_VIDEOEXPOSE:
        rendering(0);
        cleanRedrawEvent();
        break;
          
      case SDL_VIDEORESIZE:
        scrW = e.resize.w;
        scrH = e.resize.h;
        SDL_SetVideoMode(scrW, scrH, 0, SDL_OPENGL|SDL_RESIZABLE);
        initGl();
        redraw(); 
        break;
        
      case SDL_MOUSEMOTION:
        if (wait4confirm) break;
        if (e.motion.state & SDL_BUTTON(3) ) {
          float factor=1.6; // put to 1.0 for 1:1 scrolling
          eyex-=factor*(2.0*e.motion.xrel/(eyez*scrH));
          eyey+=factor*(2.0*e.motion.yrel/(eyez*scrH));
          redraw();
        }
        if ( (e.motion.state & SDL_BUTTON(2) ) ) {
          if (curSpline!=-1 && e.motion.yrel!=0) {
            if (SDL_GetModState()&KMOD_SHIFT) {
              spline[ curSpline ].IncrWidth(-e.motion.yrel);
            } else
            {
              spline[ curSpline ].IncrSize(1.0-e.motion.yrel*0.01);
            }
            doneSomething=true;
            redraw();
          }
        }
        
        if ( (e.motion.state & SDL_BUTTON(1) ) ) {
          if (curSpline==-1) {
            newSpline.Set(clicked,GetPos(e.button.x, e.button.y));
            editingNewSpline=true;
            redraw();
          }
          if ((curSpline!=-1) && (curNode!=-1)) {
            spline[ curSpline ].SetNodePos( GetPos(e.button.x, e.button.y), curNode );
            spline[ curSpline ].ArrangeLetters();
            doneSomething=true;
            redraw();
          }
        }
        break; 
       
       case SDL_MOUSEBUTTONUP:
        if (wait4confirm) break; 
        if (editingNewSpline) { 
          curSpline=spline.size();
          spline.push_back(newSpline);
          doneSomething=true;
        }
        curNode=-1;
        editingNewSpline=false;
        redraw();
        break;
       case SDL_MOUSEBUTTONDOWN:
        if (e.button.button==SDL_BUTTON_LEFT ) {
          clicked=GetPos(e.button.x, e.button.y);
          SelectSpline(clicked);
        }
        
        if (e.button.button==SDL_BUTTON_WHEELUP ) {
          eyez*=1.2;
        };
        if (e.button.button==SDL_BUTTON_WHEELDOWN ) {
          eyez/=1.2;
         };
         redraw();
        break;
     }
    } else {
      // nessun evento: siamo IDLE
      
      //Time timeNow=SDL_GetTicks(); // che ore sono?
    }
  }
  SDL_Quit();
  return(0);
}
