
#include <vcg/space/point2.h>
#include <stdio.h>
#include <strings.h>
#include <vector>

#include "letter.h"
#include "spline.h"

#include "msgGUI.h"

#include <GL\gl.h>

#include <wrap/gl/space.h>
#include <math.h>

const int DefaultAlpha=35;

vcg::Point2f Spline::Center(){
   return (v[0]+v[3])*0.5;
   /*Pos(0.5);*/ 
}


int Spline::GetNIcons(){
  int res=0;
  for (int i=0; i<letter.size(); i++) if (letter[i].c!=' ') res++;
  return res;
}

void Spline::Flip(){
  vcg::Point2f tmp;
  tmp=v[0]; v[0]=v[3]; v[3]=tmp;
  tmp=v[1]; v[1]=v[2]; v[2]=tmp;
  
  v[1].Y()*=-1;
  v[2].Y()*=-1;
  v[1].X()=1-v[1].X();
  v[2].X()=1-v[2].X();
  //float t;
  //t=v[1].Y(); v[1].Y()=v[2].Y(); v[2].Y()=t;
  ArrangeLetters();
}

char* Spline::GetString() const{
  static char res[1024];
  int i;
  for (i=0; i<letter.size(); i++) res[i]=letter[i].c;
  res[i]=0;
  return res;
}

void Spline::Load(FILE *f, int ver){
  int t;
  letter.clear();
  while (1) {
    static int safety=0; if (safety++>10000) break; 
    char c;
    fscanf(f,"%c ",&c);
    if (c=='|') break;
    letter.push_back(Letter( (c=='_')?' ':c ));
  }
  for (int i=0; i<4; i++) {
   fscanf(f, " %f %f ",&(v[i].X()), &(v[i].Y()) );
  }
  if (ver==0) 
    fscanf(f, " %f %d ",&lettersize, &alpha );
  else
    fscanf(f, " %f %d %d %d %d ",&lettersize, &alpha,&width,&italic,&colindex );
  
  GetReady();
  ArrangeLetters();
}

bool IsUC(char c){
  return ((c>='A')&&(c<='Z'));
}

bool IsLC(char c){
  return ((c>='a')&&(c<='z'));
}

char GetUC(char c){
  if (IsLC(c)) return c-'a'+'A'; else return c;
}

char GetLC(char c){
  if (IsUC(c)) return c-'A'+'a'; else return c;
}

void Spline::ChangeAllCaps() {
  bool M=false,m=false;
  for (int i=0; i<letter.size(); i++) {
    if (IsUC(letter[i].c)) M=true;
    if (IsLC(letter[i].c)) m=true;
  }
  if (M && m) { // is MIX. go to all UP
    for (int i=0; i<letter.size(); i++) letter[i].c=GetUC(letter[i].c);
  }
  if (M && !m) { // is all UP. go to all DOWN
    for (int i=0; i<letter.size(); i++) letter[i].c=GetLC(letter[i].c);
  }
  if (m && !M) { // is all DOWN. go to MIX
    bool useUC=true;
    for (int i=0; i<letter.size(); i++) {
      if (letter[i].c==' ') { useUC=true; continue;}
      letter[i].c = (useUC)?GetUC(letter[i].c):GetLC(letter[i].c);
      useUC=false;
    }
  }
}
float Spline::GetRadius() const{
  vcg::Point2f c=(v[0]-v[3])/2.0;
  c[0]=fabs(c[0]);
  c[1]=fabs(c[1]);
  if (c[0]>c[1]) return c[0]; else return c[1];
  //return (v[0]-v[3]).Norm()/2.0;
}

void Spline::Save(FILE *f){
  for (int i=0; i<letter.size(); i++) fprintf(f,"%c ",(letter[i].c==' ')?'_':letter[i].c);
  fprintf(f,"| ");
  for (int i=0; i<4; i++) {
   fprintf(f, " %f %f ",v[i].X(), v[i].Y() );
  }
  fprintf(f, " %f %d %d %d %d\n",lettersize,alpha, width,italic,colindex );
}

void DrawHandle(vcg::Point2f a, bool selected){
  if (!selected) glColor3f(0.9,0.6,0); else glColor3f(1,0.8,0);
  float s=0.3;
  if (selected) s=0.4;
  glBegin(GL_QUADS);
  glVertex2d(a[0]+s,a[1]);
  glVertex2d(a[0],a[1]+s);
  glVertex2d(a[0]-s,a[1]);
  glVertex2d(a[0],a[1]-s);
  glEnd();
}

vcg::Point2f Spline::Ori(float q){
  float t=1-q;
  return (
     vw[0]*(t*t*3.0) 
    +vw[1]*(t*(2-3*t)*3.0) 
    +vw[2]*((1-4*t+3*t*t)*3.0) 
    -vw[3]*(q*q*3.0)).Normalize();  
}

vcg::Point2f Spline::Pos(float q){
  float t=1-q;
  return 
    vw[0]*(t*t*t) +
    vw[1]*(t*t*q*3.0) +
    vw[2]*(t*q*q*3.0)+
    vw[3]*(q*q*q);  
}

Spline::Spline(){
  lettersize=1.0; 
  alpha=DefaultAlpha;
  italic=false;
  tmpwidth=width=100;
  colindex=0;
}

void Spline::CopyStats(const Spline& sp){
  lettersize=sp.lettersize;
  alpha=sp.alpha;
  italic=sp.italic;
  tmpwidth=width=sp.width;
  colindex=sp.colindex;
}

void Spline::IncrWidth(int howmuch){
  int K=15;
  
  tmpwidth+=howmuch;
  if (tmpwidth<100-K) width=tmpwidth+K; 
  else if (tmpwidth>100+K) width=tmpwidth-K;
  else width=100;
  
  //width+=howmuch;
  if (width<40) width=40;
  if (width>250) width=250;
  
  UpdateLetterStats(); 
}

void Spline::IncrAlpha(int howmuch){
  alpha+=howmuch;
  if (alpha<10) alpha=10;
  if (alpha>100) alpha=100;
  UpdateLetterStats(); 
}

void Spline::IncrSize(float howmuch){
  if (howmuch<0.5) howmuch=0.5;
  lettersize*=howmuch;
  UpdateLetterStats();
}

void Spline::UpdateLetterStats(){
  for (int i=0; i<letter.size(); i++) {
    letter[i].size=lettersize;
    letter[i].alpha=(alpha*255)/100; 
    letter[i].italic=italic;
    letter[i].width=width;
  } 
}


void Spline::ArrangeLetters(){
  GetReady();
  int n=letter.size();

  // compute vector of distances (for evenly spaced letters)
  const int NSEG=10;
  float dist[NSEG+1];
  float curD=0.0, lenght;
  vcg::Point2f curP=Pos(0); //vw[0]; 
  dist[0]=0;
  for (int i=1; i<=NSEG; i++) {
    vcg::Point2f nextP=Pos(i/(float)NSEG);
    curD+=(curP-nextP).Norm();
    curP=nextP;
    dist[i]=curD;
  }
  lenght=curD;

  for (int i=0; i<n; i++){
    float t=(0.5+i)/(n);
    
    // correct t (for evenly spaced letters)
    float tl=t*lenght;
    int j=1; while (dist[j]<=tl && j<NSEG) j++;
    float k=0; 
    if (dist[j]-dist[j-1]!=0) k=(tl-dist[j-1])/(dist[j]-dist[j-1]);
    float tCorr= (j-1+k)/(NSEG);
    
    letter[i].pos=Pos(tCorr);
    letter[i].ori=Ori(tCorr);
  }
  
  UpdateLetterStats();
}

void Spline::AddLetter(char c){
  letter.push_back(Letter(c));
  ArrangeLetters();
}

bool Spline::RemoveLetter(){
  
  //char msg[10]; sprintf(msg,"from %d",letter.size());Msg::Add(msg);
  if (letter.size()==0) return false;
  letter.resize(letter.size()-1);
  ArrangeLetters();
  //sprintf(msg,"to %d",letter.size());Msg::Add(msg);
  return true;
}

void Spline::SetNodePos(vcg::Point2f x, int i){
  if (i==0 || i==3) v[i]=x;
  else {
    vcg::Point2f a=v[3]-v[0];
    a/=a.SquaredNorm();
    vcg::Point2f b(a[1],-a[0]);
    v[i].X()=a*(x-v[0]);
    v[i].Y()=b*(x-v[0]);
  }
  GetReady();
}

void Spline::GetReady(){
  vw[0]=v[0];
  vw[3]=v[3];
  
  vcg::Point2f a=v[3]-v[0], b(a[1],-a[0]);
  vw[1]= v[0] + a*(v[1].X()) + b*(v[1].Y());
  vw[2]= v[0] + a*(v[2].X()) + b*(v[2].Y()); 
}

void Spline::Set(vcg::Point2f a, vcg::Point2f b){
  v[0]=a;
  v[3]=b;
  v[1]=vcg::Point2f(0.33,0);
  v[2]=vcg::Point2f(0.66,0);
}


float Spline::Dist(vcg::Point2f x){
  float min2=1000;
  for (float t=0; t<1.0; t+=0.02) {
    float d=(Pos(t)-x).Norm();
    if (d<min2) min2=d;
  }
  return min2;
}; 

float Spline::NodeDist(vcg::Point2f x, int i){
  return (vw[i]-x).Norm();
};

void Spline::Render(bool selected, int selNode){
  GetReady();
  if (selected) glColor3f(0,0,0); else glColor3f(.5,.5,.5);
  
  glBegin(GL_LINE_STRIP);
  for (float t=0; t<1.0; t+=0.02) {
    vcg::glVertex(Pos(t));
  }
  glEnd();
  
  if (selected) {
    for (int i=0; i<4; i++) DrawHandle(vw[i], (i==selNode) );
  
    glColor3f(0.9,0.6,0);
    glBegin(GL_LINES);
      for (int i=0; i<4; i++) vcg::glVertex(vw[i]);
    glEnd();
  }

  glEnable(GL_TEXTURE_2D); 
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
  for (int i=0; i<letter.size(); i++) letter[i].Render();
  glDisable(GL_BLEND);
  glDisable(GL_TEXTURE_2D);
}

